/* testDll.c : A small test dll
 * Tests:
 *   - Linkage to global vars in zkl (a compile & run time check)
 * To Use:
 *   - var t = Import("testDll")
 *     t.test() --> True or False depending on the test results
 */

#include <stdio.h>
#include <string.h>

#include "zklObject.h"
#include "zklClass.h"
#include "zklData.h"
#include "zklFcn.h"
#include "zklList.h"
#include "zklNumber.h"
#include "zklString.h"
#include "zklImports.h"

static ZKL_Object TestDllObject;

/* ******************************************************************** */
/* ***************************** Methods ****************************** */
/* ******************************************************************** */

static int result;

static void compare(char *name, Instance *guess, Instance *value)
{
   if (guess != value)
   {
      printf("Opps: Bad pointer to %s\n",name);
      result = 0;
   }
}

    // .testGlobals --> Bool
static Instance *testGlobals(Instance *self,pArglist arglist,pVM vm)
{
   result = 1;
   printf("Checking global vars\n");

   compare("String",emptyString,stringCreate("",I_OWNED,vm));
   compare("String",Star,stringCreate("*",I_OWNED,vm));

   compare("Bool",BoolTrue,boolCreate(1));
   compare("Bool",BoolFalse,boolCreate(0));
   compare("Void",Void,voidCreate());

   compare("Tuple",emptyTuple,tupleCreate(0,I_OWNED,vm));
   compare("Tuple",emptyTuple,NoArglist);

   compare("Int",Zero,intCreate(0,vm)); compare("Int",One,intCreate(1,vm));
   compare("Float",FZero,floatCreate(0.0,vm));
   compare("Float",FOne,floatCreate(1.0,vm));

   compare("Object table",
      (Instance *)&TestDllObject,(Instance *)zklObjectTable[self->objID]);

   if (strcmp("NullClass",className(NullClass)))
   {
      printf("NullClass not good\n");
      result = 0;
   }

   if (strcmp("nullFcn",fcnName(nullFcn)))
   {
      printf("nullFcn not good\n");
      result = 0;
   }

   if (result) printf("Global vars: All good\n");

   return boolCreate(result);
}

    // Test for the possibility of two CRTs (C Run Times)
    // Test passing between DLLs
    // http://msdn.microsoft.com/en-US/library/ms235460.aspx
    // This will fail if testDll and zkl are compiled /MT (vs /MD)
    //   (zkl /MD & testDll /MT OK, vice versa not)
    // test.zkl will see if "zklTestDll" is set outside Dll
static Instance *testEV(Instance *self,pArglist arglist,pVM vm)
{
   #ifdef _MSC_VER
      _putenv("zklTestDll=test");   
   #else
      putenv("zklTestDll=test");   
   #endif
      return tupleCreateX(vm,
         stringCreate("zklTestDll",I_OWNED,vm),stringCreate("test",I_OWNED,vm),
	 ZNIL);
//   #else
//      return BoolTrue;
//   #endif
}

FILE *fileFILE(Instance *);	// file.c

    // File handles crossing Dll boundries
    // .testFile(File)
    // Core dumper if mislinked
static Instance *testFile(Instance *self,pArglist arglist,pVM vm)
{
   char *text = "This is a Dll file test";
   FILE *file = fileFILE(arglistGet(arglist,0,0,vm));
   if (!file) return Void;
   fprintf(file,"%s",text);
   return stringCreate(text,I_OWNED,vm);
}

   // Malloc here, free over there
static Instance *testMalloc(Instance *self,pArglist arglist,pVM vm)
{
   char  text[] = "Dll malloc test data";
   int	 size = sizeof(text) + 1;
   char *ptr  = malloc(size);
   strcpy(ptr,text);
   return kdataCreate((Byte *)ptr,size,(Instance *)1,I_OWNED,vm);
}


static MethodTable methods[] =
{
   "testGlobals",	(pMethod)testGlobals,
   "testEV",		(pMethod)testEV,
   "testFile",		(pMethod)testFile,
   "testMalloc",	(pMethod)testMalloc,
   0,		0
};


/* ******************************************************************** */
/* **************************** Properties **************************** */
/* ******************************************************************** */

Instance testInstance;

DllExport void *construct(void *vm)
{
   if (!vm) return (void *)ZKLX_PROTOCOL;	// handshake
   	// If this is reloaded, nothing happens except 
	// construct() is called again so don't reinitialize
   if (!TestDllObject.name)	// static items are zero
   {
      constructObject(&TestDllObject,NativeType, methods,0,0,vm);
      TestDllObject.name      = "testDll";

      instanceInit(&testInstance,&TestDllObject,I_UNTOUCHABLE);
   }
   return &testInstance;
}
